/* SECU-3  - An open source, free engine control unit
   Copyright (C) 2007 Alexey A. Shabelnikov. Ukraine, Gorlovka

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   contacts:
              http://secu-3.org
              email: shabelnikov@secu-3.org
*/

#include "stdafx.h"
#include "ComPort.h"

//warning C4800: 'int' : forcing value to bool 'true' or 'false' (performance warning)
#pragma warning( disable : 4800 )

//=============================================================================
// Constructor
ComPort::ComPort()
: rxQueue(32768)
, txQueue(32768)
, commPort(INVALID_HANDLE_VALUE)
{
 //empty
}

//=============================================================================
// Destructor
ComPort::~ComPort()
{
 if (commPort != INVALID_HANDLE_VALUE)
  CloseHandle (commPort);
}

//=============================================================================
// Opens the specified com port.
bool ComPort::open(LPCTSTR port, DWORD baudRate, DWORD timeOut)
{
 // Make sure that channel is closed
 if (commPort != INVALID_HANDLE_VALUE)
  return false;

 // Open com port
 commPort = CreateFile(port, GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
 if (commPort == INVALID_HANDLE_VALUE)
  return false;

 // Communication buffers
 if (!SetupComm(commPort, rxQueue, txQueue))
 {
  CloseHandle(commPort);
  return false;
 }

 // Set up the serial communications device
 DCB dcb;
 ZeroMemory (&dcb, sizeof(DCB));
 dcb.DCBlength = sizeof(DCB);
 dcb.BaudRate = baudRate;
 dcb.fBinary = 1;
 dcb.fDtrControl = DTR_CONTROL_DISABLE; //DTR
 dcb.fRtsControl = RTS_CONTROL_ENABLE;  //RTS
 dcb.ByteSize = 8;
 dcb.Parity = NOPARITY;
 dcb.StopBits = ONESTOPBIT;

 if (!SetCommState(commPort, &dcb))
 {
  CloseHandle(commPort);
  return false;
 }

 if (!setTimeOut(timeOut))
 {
  CloseHandle(commPort);
  return false;
 }

 return true; //ok
}

//=============================================================================
// Sets timeouts
bool ComPort::setTimeOut(DWORD timeOut)
{
 COMMTIMEOUTS ctmo;
 ZeroMemory (&ctmo, sizeof(COMMTIMEOUTS));
 ctmo.ReadIntervalTimeout = timeOut;
 ctmo.ReadTotalTimeoutMultiplier = timeOut;
 ctmo.ReadTotalTimeoutConstant = timeOut;

 return SetCommTimeouts(commPort, &ctmo);
}

//=============================================================================
// Purges the input and output buffers of the COM port
bool ComPort::purge()
{
 if (commPort == INVALID_HANDLE_VALUE)
  return false;

 if (!PurgeComm(commPort, PURGE_TXCLEAR | PURGE_RXCLEAR))
  return false;

 return true;
}

//=============================================================================
// Reads 'i_size' bytes from the COM port
bool ComPort::read(UCHAR *o_buffer, DWORD i_size)
{
 // Return if com port handler is not valid
 if (commPort == INVALID_HANDLE_VALUE)
  return false;

 DWORD read;
 if (!ReadFile(commPort, o_buffer, i_size, &read, NULL))
  return false;

 return (read == i_size);
}

//=============================================================================
// Writes 'i_size' bytes to the COM port
bool ComPort::write(UCHAR *i_buffer, DWORD i_size)
{
 // Return if com port handler is not valid
 if (commPort == INVALID_HANDLE_VALUE)
  return false;
	
 DWORD written;
 if (!WriteFile (commPort, i_buffer, i_size, &written, NULL))
  return false;

 return (written == i_size);
}

//=============================================================================
// Returns handle to COM port
HANDLE ComPort::getHandle(void) const
{
 return commPort;
}

//=============================================================================
// Closes the com port
bool ComPort::close()
{
 if (commPort != INVALID_HANDLE_VALUE)
  CloseHandle (commPort);

 commPort = INVALID_HANDLE_VALUE;
 return true;
}
