/* SECU-3  - An open source, free engine control unit
   Copyright (C) 2007 Alexey A. Shabelnikov. Ukraine, Gorlovka

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.

   contacts:
              http://secu-3.org
              email: shabelnikov@secu-3.org
*/

#include "stdafx.h"
#include "comport.h"
#include <iostream>
#include <ctype.h>

unsigned char buff[32768];
unsigned char buff1[32768];

BYTE HTOD(BYTE h)
{
 return ((h<0x3A)?h-'0':h-'A'+10);
}

bool HexArrayToBin(const BYTE* i_buf, BYTE* o_buf, const int size)
{
 int j=0;
 for(int i = 0; i < size; i++)
 {
  if (isxdigit(i_buf[j])&&(isxdigit(i_buf[j+1])))
  {
   o_buf[i] = HTOD(i_buf[j])*16 + HTOD(i_buf[j+1]);
  }
  else
   return false;  //error: not a HEX digit!
  j+=2;
 } //for
 return true;
}

BYTE CheckSum_8_xor(const BYTE* i_buf,const int size)
{
 BYTE sum=0;
 for(int i = 0; i < size; i++)
 {
  sum = sum ^ i_buf[i]; //XOR
 }//for
 return sum;
}

//interface commands
#define CMD_EXIT_BL "!T"
#define CMD_READ_EE "!J"
#define APP_STRT_BL "!il\r"

//entry point
int main(int argc, char* argv[])
{
 --argc;
 if (argc != 3)
 {
  std::cout << std::endl << "Invalid arguments!" << std::endl;
  std::cout << std::endl << "Usage: " << argv[0] << " port speed eeprom_size" << std::endl;
  return 7;
 }

 bool result;
 const char* portName = argv[1];  //COM port
 DWORD speed = atoi(argv[2]);     //Speed
 UINT eepromSize = atoi(argv[3]); //Size of EEPROM

 std::cout << "port=" << portName << " speed=" << speed << " eepromSize=" << eepromSize << std::endl;
 
 ComPort p;
 p.open(portName, speed, 200);

 p.write((unsigned char*)APP_STRT_BL, strlen(APP_STRT_BL));
 Sleep(100); //wait while data will be transferred

 p.purge(); //clear remaining ingoing packets
 
 std::cout << std::endl << "Boot loader successfully started!" << std::endl;

 p.write((unsigned char*)CMD_READ_EE, strlen(CMD_READ_EE));
 
 if (false==p.read(buff, 1) || buff[0] != '<') //read '<'
  result = false;
 
 UINT datasize = eepromSize * 2;  // EEPROM size
 if (false==p.read(buff, datasize)) //read data
  result = false;

 if (false==p.read(buff1, 2))  //CS
  result = false;

 BYTE chksum_read = HTOD(buff1[0]) * 16 + HTOD(buff1[1]);

 //////////////////////////////////////////////////////////////////////////////
 std::cout << "Size= " << datasize << " Data:" << std::endl;
 for (size_t i = 0; i < datasize; ++i)
 {
  char str[2] = {buff[i], 0};
  std::cout << str;
  //if (i==0) continue;

  if (0==((i+1) % (2*16)))
   std::cout << std::endl;
  else if (0==((i+1) % 2) )
   std::cout << " ";
 }
 std::cout << std::endl;
 //////////////////////////////////////////////////////////////////////////////

 HexArrayToBin(buff, buff1, eepromSize);
 BYTE chksum_calc = CheckSum_8_xor(buff1, eepromSize);

 if (false == result || chksum_read != chksum_calc)
 {
  std::cout << std::endl << "02 Error reading EEPROM" << std::endl;
  std::cout << "chksum=" << std::hex << (unsigned)chksum_read << "/" << (unsigned)chksum_calc << std::endl;
 }
 else
 {
  std::cout << std::endl << "Successfully read EEPROM data!" << std::endl;
 }

 p.write((unsigned char*)CMD_EXIT_BL, strlen(CMD_EXIT_BL));
 result = p.read(buff, 2);

 if (false == result && buff[0] != '<' && buff[1] != '@')
 {
  std::cout << std::endl << "01 Error exiting from boot loader!" << std::endl;
  return 1;
 }
 else
  std::cout << std::endl << "Successfully exited from boot loader!" << std::endl;


 p.close();
	return 0;
}

